package de.adwin.examples;

import java.util.*;
import java.io.IOException;
import java.io.InputStream;
import java.io.File;
import java.io.FileReader;
import java.net.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*; 

import de.adwin.driver.*;

public class Demo extends JApplet implements ActionListener, FocusListener
{
    class DemoTimerTask extends TimerTask
    {
        public void run()
        {
            // System.out.println("Timer");
        
            try
            {
    	        if (fADwinBooted == true)
    	        {
    	            if (fTextfieldPAR1.hasFocus() == false)
    	            {    
    	                fTextfieldPAR1.setText(Integer.toString(fADwin.Get_Par(1)));
    	            }
    	        }
            }
            catch (ADwinCommunicationError ex)
            {
                fADwinBooted = false;
                System.out.println(ex);
            }
        }
    }
    
    private JTextField fTextfieldIP;
    private JTextField fTextfieldPAR1;    
    private JRadioButton fRadioT9;
    private JRadioButton fRadioT10;
    private JRadioButton fRadioT11;    
    
    private ADwinDevice fADwin;
    private boolean fADwinBooted;
    
    public void init() 
    {
        showUIapplet();
    }

    public void start() 
    {
    }

    public void stop() 
    {
    }

    public void destroy() 
    {
    }
    
    public void focusLost(FocusEvent e)
    {
    }
    
    public void focusGained(FocusEvent e)
    {
    }
    
    public void actionPerformed(ActionEvent e)
    {
        // boot
    	
    	String basePathName = "";
        
        if (e.getActionCommand().equals("boot"))
        {
        	// This is important: As the Timer that updates the PAR_1 text field runs on its own thread,
        	// it could try to read the parameter while the following code tries to boot the system.
        	// In more complicated cases (where setting a single flag would not help), use the
        	// ADwinDevice.Get_Copy () method to create 2 objects that do not disturb each other. 
        	fADwinBooted = false;
        	// the following code first tries to load a matching BTL file from disk (depending on the
        	// underlaying OS, there are many possibilities where they can be).
        	// If it does not work, the code assumes it is in a JAR file, tries to load the
        	// BTL as a resource and uses an undocumented version of the Boot method to boot the ADwin system
            try
            {
                if (fADwin == null)
                {
                    fADwin = new ADwinDevice();
                }
                
                String ip = fTextfieldIP.getText(); 
                if (!ip.equals(""))
                {
	                fADwin.Remove_Entry(0x01);
		            fADwin.Add_Entry(0x01, InetAddress.getByName(ip), "", 1000, 6543);		
		            fADwin.Set_DeviceNo(0x01);
		            
		    		if (new File ("/opt/adwin/share/btl").exists ())	// See where the BTL files are....
		    			basePathName = "/opt/adwin/share/btl/";			// (Linux old)
		    		else if (new File ("/etc/adwin/ADWINDIR").exists ())
		    		{													// (Linux new)
		    			try
		    			{
		    				FileReader fr = new FileReader ("/etc/adwin/ADWINDIR");
		    				char addir[] = new char[500];
		    				fr.read (addir, 0, 500);
		    				fr.close ();
		    				int n = 0;
		    				while (addir[n] != 0 && addir[n] != '\n' && addir[n] != 10)
		    					basePathName += addir[n++];
		    				basePathName += "/share/btl/";
		    				
		    			}
		    			catch (IOException ioe)
		    			{
		    				JOptionPane.showMessageDialog  (this, "I/O-Error when reading from /etc/adwin/ADWINDIR -\n" + ioe.getMessage (), "Error", JOptionPane.ERROR_MESSAGE);
//		    				new ErrorFrame ("I/O-Error when reading from /etc/adwin/ADWINDIR -\n" + ioe.getMessage ());
		    				return;
		    			}
		    		}
		    		else 
		    			basePathName = "c:\\adwin\\";					// Windows
	        		
		    		try
					{
		    			if (fRadioT9.isSelected ())	 					// T9
		    				fADwin.Boot (basePathName + "adwin9.btl");
		    			else if (fRadioT10.isSelected ())				// T10
		    				fADwin.Boot (basePathName + "adwin10.btl");
		    			else											// T11
		    				fADwin.Boot (basePathName + "adwin11.btl");
					}
		    		catch (ADwinCommunicationError ace)
					{
		    			// couldn't load the file from disk, so try to load as a resource
		    			if (ace.getCode () != 200) throw ace;
			            InputStream is = null;
			            URL resURL = null;
			            try
						{
			            	String BTLFileName;
			            	if (fRadioT9.isSelected() == true) // T9
			            		resURL = getClass ().getClassLoader ().getResource (BTLFileName = "adwin9.btl");
			            	else if (fRadioT10.isSelected ())	// T10
			            		resURL = getClass ().getClassLoader ().getResource (BTLFileName = "adwin10.btl");
			            	else								// T11
			            		resURL = getClass ().getClassLoader ().getResource (BTLFileName = "adwin11.btl");
			            
			            	if (resURL == null)
			            	{
			    				JOptionPane.showMessageDialog  (this, "BTL files could neither be found on your disk nor in the JAR archive\n" +
			    								"Please check your ADwin installation", "Error", JOptionPane.ERROR_MESSAGE);
			    				return;
			            	}
			            	is = resURL.openStream ();
			            	byte BTL[] = new byte[is.available ()];
			            	int bytesRead = 0, bytesReadThisTime = 0;
			            	
			            	do
			            	{
			            		bytesReadThisTime = is.read (BTL, bytesRead, BTL.length - bytesRead);
			            		bytesRead += bytesReadThisTime;
			            	}
				            while (bytesReadThisTime > 0);
				            		
			            	is.close ();
			            	
			            	// Function is not officially documented. The second parameter is simply
			            	// used to generate an error message.
			            	fADwin.Boot (BTL, BTLFileName);
				            	
						}
			            catch (IOException ioe)
						{
		    				JOptionPane.showMessageDialog  (this, "I/O-Error reading from JAR file -\n" + ioe.getMessage (), "Error", JOptionPane.ERROR_MESSAGE);
		    				return;
						}
					}
		            fADwinBooted = true;
                }
            }
            catch (UnknownHostException ex)
            {
				JOptionPane.showMessageDialog  (this, ex, "Error", JOptionPane.ERROR_MESSAGE);
            }
            catch (ADwinCommunicationError ex)
        	{
				JOptionPane.showMessageDialog  (this, ex, "Error", JOptionPane.ERROR_MESSAGE);
        	}
        }
        else if (e.getActionCommand().equals("SetPar"))
        {
            if (fADwinBooted == true)
            {
                try
                {
                    fADwin.Set_Par(1, Integer.parseInt(fTextfieldPAR1.getText()));
                }
                catch (ADwinCommunicationError ex)
                {
    				JOptionPane.showMessageDialog  (this, "Error when setting PAR_1 - " + ex.getMessage (), "Error", JOptionPane.ERROR_MESSAGE);
                    fADwinBooted = false;
                }
            }
        }
    }  
    
    
    private void showUIapplet()
    {
        Container cp = this.getContentPane();
        showUI(cp);
    }
    
    private void showUIapplication()
    {
        JFrame.setDefaultLookAndFeelDecorated(true); 
        JFrame frame = new JFrame();
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE); 
    
        frame.setLocation(100, 100);
        
        frame.setTitle("ADwin Java Demoprogram");

        Container cp = frame.getContentPane();
        showUI(cp);
        
        //Display the window.
        frame.pack();
        frame.setVisible(true);        
    }
    
    private void showUI(Container cp)
    {
        fADwinBooted = false;
        
        //
        
        GridBagLayout gb = new GridBagLayout();
        GridBagConstraints gbc = new GridBagConstraints();
        Insets insets = new Insets(4, 4, 4, 4);
        

        cp.setLayout(gb);
        
        //
        
        // GridBagConstraints(int gridx, int gridy, int gridwidth, int gridheight, double weightx, double weighty, int anchor, int fill, Insets insets, int ipadx, int ipady) 

        JLabel label = new JLabel("IP address");
        cp.add(label, new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0, GridBagConstraints.EAST, GridBagConstraints.NONE, insets, 0, 0));
        
        fTextfieldIP = new JTextField(16);     
        cp.add(fTextfieldIP, new GridBagConstraints(1, 0, 2, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.NONE, insets, 0, 0));

        //
        
        label = new JLabel("Processor");
        cp.add(label, new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0, GridBagConstraints.EAST, GridBagConstraints.NONE, insets, 0, 0));
        	
        JPanel panel = new JPanel();
        	
        fRadioT9 = new JRadioButton("T9");
        fRadioT9.setActionCommand("T9");
        fRadioT9.addActionListener(this);
        fRadioT9.setSelected(true);
        panel.add(fRadioT9);
        
        fRadioT10 = new JRadioButton("T10");
        fRadioT10.setActionCommand("T10");
        fRadioT10.addActionListener(this);
        panel.add(fRadioT10);

        fRadioT11 = new JRadioButton("T11");
        fRadioT11.setActionCommand("T11");
        fRadioT11.addActionListener(this);
        panel.add(fRadioT11);

        ButtonGroup group = new ButtonGroup();
        group.add(fRadioT9);
        group.add(fRadioT10);
        group.add(fRadioT11);
        
        cp.add(panel, new GridBagConstraints(1, 1, 2, 1, 1.0, 1.0, GridBagConstraints.WEST, GridBagConstraints.NONE, insets, 0, 0));
        
        //
        
        //label = new JLabel("");
        //cp.add(label, new GridBagConstraints(0, 2, 1, 1, 1.0, 1.0, GridBagConstraints.EAST, GridBagConstraints.NONE, insets, 0, 0));

        JButton button = new JButton("Boot");     
        button.setActionCommand("boot");
        button.addActionListener(this);
        cp.add(button, new GridBagConstraints(1, 2, 2, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, insets, 0, 0));
        
        //
        
        label = new JLabel("PAR_1");
        cp.add(label, new GridBagConstraints(0, 3, 1, 1, 1.0, 1.0, GridBagConstraints.EAST, GridBagConstraints.NONE, insets, 0, 0));
        
        fTextfieldPAR1 = new JTextField(10);     
        fTextfieldPAR1.addFocusListener(this);
        cp.add(fTextfieldPAR1, new GridBagConstraints(1, 3, 1, 1, 1.0, 1.0, GridBagConstraints.WEST, GridBagConstraints.NONE, insets, 0, 0));
        
        JButton jb = new JButton ("Set");
        jb.setActionCommand ("SetPar");
        jb.addActionListener (this);
        cp.add (jb, new GridBagConstraints (2, 3, 1, 1, 1.0, 1.0, GridBagConstraints.WEST, GridBagConstraints.NONE, insets, 0, 0));
        
        //
        
        
        DemoTimerTask demott = new DemoTimerTask();
        
        java.util.Timer timer = new java.util.Timer();
        timer.schedule(demott, 0, 1000);        
    }
    
    public static void main(String[] args)
    {
        //System.out.println("Start main");
        
        Demo demo = new Demo();
        demo.showUIapplication();

        //System.out.println("End main");
    }
    
} // class DEMO
